/*
 * Constantin Sorin 332CC
 * PAC.c
 */

#include "lcd.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>


#define DIST_GET_BTN PB0
#define DIST_DEL_BTN PB1
#define AREA_DEL_BTN PB2

static volatile unsigned int phase = 0; /* nivel logic */
static volatile unsigned int count = 0; /* numar pulsuri */

unsigned int lungime = 0;
unsigned int diagonala = 0;
unsigned int arie = 0;
unsigned int arie_totala = 0;
unsigned int nr_mas = 0;

/* rutina de intreruperi la schimbarea de nivel logic */
ISR(PCINT0_vect)
{
	if (phase == 0) {
		/* enable counter */
		TCCR1B |= (1 << CS11);
		phase = 1;
	} else if (phase == 1) {
			/* reset and disable counter */
			TCCR1B = 0;
			count = TCNT1; /* get the count range */
			TCNT1 = 0;
			phase = 0;
	}
}

/* compute area knowing height and diagonal*/
int compute_area(int lung, int diag)
{
	double x, x2;
	x2 = pow(((double) diag), 2) - pow(((double) lung), 2);
	x = sqrt(x2);
	
	return lung*((int) x);
}


/* initializare intreruperi, display si configurare pini */
void IO_init(void)
{
	sei(); /*enable SREG*/
	
	PCICR |= (1 << PCIE0);
	PCMSK0 |= (1 << PCINT0);
	
	LCD_init();
	
	
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);
	
	DDRB |= (1 << PB3);
	DDRB &= ~((1 << PB0) | (1 << PB1));
	PORTB |= (1 << PB0) | (1 << PB1);
	
	DDRD |= (1 << PD5);     // PD5 trebuie sa fie output
	PORTD |= (1 << PD5);
}

/* redare ton buzzer */
void speaker_morse(int tip)
{
    int delay = 1;
    int i;
    
    for (i = 0; i < tip * 50 / delay; i++)
    {
        PORTD ^= (1 << PD5); // alternam starea iesirii
        _delay_ms(1);
    }
}
/* return pressed button code */
int get_button_pressed(void)
{
	/*
		0 - PB0 (DIST_GET_BTN)
		1 - PB1 (DIST_DEL_BTN)
		2 - PB2 (AREA_DEL_BTN)
	*/
	if ((PINB & (1 << DIST_GET_BTN)) == 0)
	{
		PORTD |= (1 << PD7);
		_delay_ms(10);
		PORTD &= ~(1 << PD7);
		
		_delay_ms(200);
		return 0;
	}
	else if ((PINB & (1 << DIST_DEL_BTN)) == 0)
		{
			_delay_ms(200);
			return 1;
		}
		else if ((PINB & (1 << AREA_DEL_BTN)) == 0)
			{
				_delay_ms(200);
				return 2;
			}
			else 
			{
				return -1;
			}
}


/* print distance on lcd */
void print_distance(int distance, int type)
{
	char distBuf[16], pac[] = "PAC!", cm[] = "cm", lat[] = "Lat: ", diag[] = "Diag: "; 
	PORTD |= (1 << PD5);
	switch (type)
	{
		case 0: 	
			if (distance>0 && distance<=300)
			{
				LCD_writeInstr(LCD_INSTR_clearDisplay);	
				itoa(distance, distBuf, 10);
				LCD_printAt(2, pac, 1); /* Tupac FTW !!! */
				LCD_printAt(9, distBuf, 1);	
				LCD_printAt(13, cm, 1);
				_delay_ms(5);
				break;
			}
			else
			{
				LCD_writeInstr(LCD_INSTR_clearDisplay);	
				LCD_printAt(2, pac, 1); /* Tupac FTW !!! */
				LCD_printAt(7, ">MAXRange", 1);
				speaker_morse(1);
				_delay_ms(5);
				break;
			}
		case 1:
			if (distance>0 && distance<=300)
			{
				LCD_writeInstr(LCD_INSTR_clearDisplay);	
				itoa(distance, distBuf, 10);
				LCD_printAt(2, pac, 1); /* Tupac FTW !!! */
				LCD_printAt(2, lat, 2);
				LCD_printAt(9, distBuf, 2);	
				LCD_printAt(13, cm, 2);
				_delay_ms(5);
				break;
			}
			else
			{
				LCD_writeInstr(LCD_INSTR_clearDisplay);	
				LCD_printAt(2, pac, 1); /* Tupac FTW !!! */
				LCD_printAt(7, ">MAXRange", 1);
				speaker_morse(1);
				_delay_ms(5);
				break;
			}
		case 2:
			if (distance>0 && distance<=300)
			{
				LCD_writeInstr(LCD_INSTR_clearDisplay);	
				itoa(distance, distBuf, 10);
				LCD_printAt(2, pac, 1); /* Tupac FTW !!! */
				LCD_printAt(2, diag, 2);
				LCD_printAt(9, distBuf, 2);	
				LCD_printAt(13, cm, 2);
				_delay_ms(5);
				break;
			}
			else
			{
				LCD_writeInstr(LCD_INSTR_clearDisplay);	
				LCD_printAt(2, pac, 1); /* Tupac FTW !!! */
				LCD_printAt(7, ">MAXRange", 1);
				speaker_morse(1);
				_delay_ms(5);
				break;
			}
	}
	
}

/* find number of digits of a number */
int nr_digits(int number)
{
        int nr = 0;
        while(number){
                number /= 10;
                nr++;
        }
        return nr;
}

/* converts double in interval(1.00 <-> 99.9) to char array */
char* double_to_char_array(double number)
{
        char* converted_nr = (char*)malloc(5);
        double copy = number;
        int copy2 = 0;
        int aux = (int) number;
        int dig1 = 0, dig2 = 0;
        dig1 = nr_digits(aux);

        while(aux != copy) {
                copy*=10;
                aux = (int) copy;
                dig2++;
        }

        while(nr_digits(copy) > 3){
                copy /= 10;
        }
        copy2 = (int)copy;
        if (dig1 == 1){
              converted_nr[4] = '\0';
              converted_nr[3] = copy2%10+'0';
              copy2/=10;
              converted_nr[2] = copy2%10+'0';
              copy2/=10;
              converted_nr[1] = '.';
              converted_nr[0] = copy2%10+'0';
        } else if (dig1 == 2) {
              converted_nr[4] = '\0';
              converted_nr[3] = copy2%10+'0';
              copy2/=10;
              converted_nr[2] = '.';
              converted_nr[1] = copy2%10+'0';
              copy2/=10;
              converted_nr[0] = copy2%10+'0';
        }

        return converted_nr;

}

/* prints area in cm^2 */
void print_area(int arie)
{
	char areaBuf[16], pac[] = "PAC!", cm2[] = "cm^2", ar[] = "Arie: ";
	
	LCD_writeInstr(LCD_INSTR_clearDisplay);	
	itoa(arie, areaBuf, 10);
	LCD_printAt(2, pac, 1); /* Tupac FTW !!! */
	LCD_printAt(1, ar, 2);
	LCD_printAt(8, areaBuf, 2);
	LCD_printAt(12, cm2, 2);

	_delay_ms(5);
}

/* prints area in m^2 */
void print_areaD(double arie)
{
	char *areaBuf, pac[] = "PAC!", m2[] = "m^2", ar[] = "Arie: ";
	
	LCD_writeInstr(LCD_INSTR_clearDisplay);	
	areaBuf = double_to_char_array(arie);
	LCD_printAt(2, pac, 1); /* Tupac FTW !!! */
	LCD_printAt(2, ar, 2);
	LCD_printAt(9, areaBuf, 2);
	LCD_printAt(13, m2, 2);
	_delay_ms(50);
}

void run_work(void)
{
	int btn, distance;
	
	while (1)
	{
		/* give signal */
		PORTB |= (1 << PB3);
		_delay_us(15);
		PORTB &= ~(1 << PB3);
		
		/* compute distance */
		distance = count / (53*2);
		if (distance == 0)
			continue;
		
		btn = get_button_pressed();
		switch (btn)
		{
			case -1:
				print_distance(distance, 0);
				break;
			case 0:
				if (nr_mas == 0)
				{
					lungime = distance;
					nr_mas = 1;
					print_distance(lungime, 1);
					break;
				}
				else if (nr_mas == 1)
					{
						diagonala = distance;
						nr_mas = 0;
						arie = compute_area(lungime, diagonala);
						print_distance(diagonala, 2);
						
						lungime = 0;
						diagonala = 0;
						_delay_ms(10);
						arie_totala = arie_totala + arie;
						
						if (arie_totala > 0 && arie_totala < 9999)
						{
							print_area(arie_totala);
						}
						else if(arie_totala > 9999 && arie_totala < 999999)
							{
								print_areaD((double)arie_totala/10000);
							}
							else if (arie_totala > 999999){
								LCD_writeInstr(LCD_INSTR_clearDisplay);	
								LCD_printAt(2, "PAC!", 1); /* Tupac FTW !!! */
								LCD_printAt(2, "Area:", 2);
								LCD_printAt(7, ">MAXRange", 2);
								arie_totala = 0;
								_delay_ms(5);
							}
						arie = 0;
						break;
					}
		}
	}
}

int main()
{	
	IO_init();
	
	run_work();
	
	return 0;
}

